import ads;

real blockwidth = 1;
real blockheight = 0.5;
real arrowheight = 1;
real rowheight =  blockheight + arrowheight;


void mixarrow(int row, int col) {
    real x = col * blockwidth;
    real y = -row * rowheight - blockheight;
    path arr1 = (x + blockwidth/2, y) {S} .. {S} (x + blockwidth, y - arrowheight / 2) {S} .. {S} (x + 1.5*blockwidth, y - arrowheight); 
    path arr2 = reflect((x + blockwidth, 0), (x+blockwidth, 1)) * arr1;
    draw(arr1, Arrow);
    draw(arr2, Arrow);
}

void thruarrow(int row, int col) {
    real x = col * blockwidth;
    real y = -row * rowheight - blockheight;
    draw((x+blockwidth/2, y)--(x+blockwidth/2, y-arrowheight), Arrow);

}

void block(int row, int col, string alphabet) {
    real xbase = col * blockwidth;
    real ybase = -row * rowheight;
    if (alphabet == "...") {
        label("$\cdots$", (xbase+blockwidth/2, ybase-blockheight/2), (0,0));
        
    } else {
        if (alphabet != "EOF" && alphabet != "0")
        draw((xbase,ybase)--(xbase+blockwidth, ybase)--(xbase+blockwidth, ybase-blockheight)--(xbase, ybase-blockheight)--cycle);
        label("$"+alphabet+"$", (xbase+blockwidth/2, ybase-blockheight/2), (0,0));
    }
}

void blocks(int row ... string alphabets[]) {
    for (int i = 0; i < alphabets.length; ++i) {
        block(row, i, alphabets[i]);
    }
}

void thruarrows(int row, int col, int cnt) {
    for (int i = 0; i < cnt; ++i)
        thruarrow(row, col+i);
}

void mixarrows(int row, int col, int cnt) {
    for (int i = 0; i < cnt; i += 2)
        mixarrow(row, col+i);
}

void passlabel(int row, string lbl) {
    label("{\it " + lbl + "}", (-1, -row*rowheight-blockheight - arrowheight/2), W);
}

void mixer(real x, real y, real r=0.5) {
    draw((x-r,y-r) -- (x+r,y-r)--(x+r, y+r) -- (x-r, y+r) -- cycle, halfthick);
    draw( (x,y+r) {S} .. {E} (x+r,y), 0.5*white);
    draw( (x,y+r) -- (x,y-r), 0.5*white);
    draw( (x-r,y) {E} .. {S} (x,y-r), 0.5*white);
}

real PENTAMIXER_R  = 1;

pair pm_dir(pair c, int angle, real r=PENTAMIXER_R) {
    return shift(c)*scale(r)*dir(angle);
}
int A_IN = 180, A_OUT=0, A_COUT=90, A_CIN1=240, A_CIN2=300;
void pentamixer(pair c, real r=PENTAMIXER_R) {
    path unitcircle=E..N..W..S..cycle;
    pair d(int angle) {
        return pm_dir(c, angle, r);
    }
    draw(shift(c)*scale(r)*unitcircle);
    draw( d(A_IN)--d(A_OUT), 0.5*white);
    draw( d(A_IN){E}..{N}d(A_COUT), 0.5*white);
    draw( d(A_CIN1){dir(60)}..{E}d(A_OUT), 0.5*white);
    draw( d(A_CIN2){dir(120)}..{E}d(A_OUT), 0.5*white);
}


void pm_arrow(pair c, int angle, real length, bool out=false, real r=PENTAMIXER_R, string lbl="") {
    pair p1 = pm_dir(c, angle, r);
    pair p2 = pm_dir(c, angle, r+length);
    if (out)
        draw(p1--p2, e_arrow);
    else
        draw(p2--p1, e_arrow);
    if (lbl != "")
        label(lbl, p2, dir(angle));
}

void carry_arrow(pair mix1, pair mix2, string alphabet) {
    draw(mix1 + (0.5,0) -- mix2 - (0.5, 0), e_arrow);
    label((mix1+mix2)/2, "$"+alphabet+"$", N);
}
